package eu.mhsl.minecraft.WorldMuseum.viewableWorld;

import net.kyori.adventure.text.Component;
import net.kyori.adventure.util.TriState;
import org.bukkit.*;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.entity.Player;
import org.bukkit.generator.ChunkGenerator;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;

import java.io.File;
import java.io.IOException;
import java.util.Objects;

public class ViewableWorld {
    private final File worldLocation;
    private World world;

    private boolean enabled = true;
    private String name;
    private Material icon = Material.GRASS_BLOCK;

    public ViewableWorld(File worldLocation) {
        this.worldLocation = worldLocation;
        this.name = "unnamed_" + this.worldLocation.toPath().getFileName().toString();
        File configFile = new File(this.worldLocation.getAbsolutePath() + "/config.yml");
        if (configFile.exists()) {
            FileConfiguration config = YamlConfiguration.loadConfiguration(configFile);
            this.enabled = config.getBoolean("enabled");
            this.name = config.getString("name");
            this.icon = Material.valueOf(config.getString("icon"));
        } else {
            System.out.println("[WorldMuseum] Could not find config.yml");
            try {
                configFile.createNewFile();
                FileConfiguration data = YamlConfiguration.loadConfiguration(configFile);
                data.set("enabled", false);
                data.set("name", this.name);
                data.set("icon", this.icon.name());
                data.save(configFile);

            } catch (IOException e) {e.printStackTrace();}
        }
    }

    public void loadWorld() {
        if(world != null) throw new IllegalStateException("World was already loaded");
        WorldCreator worldCreator = new WorldCreator(this.worldLocation.getAbsolutePath());
        worldCreator.keepSpawnLoaded(TriState.FALSE);
        worldCreator.generator(new ChunkGenerator() {});
        this.world = worldCreator.createWorld();
        Objects.requireNonNull(this.world);
        this.world.setAutoSave(false);
    }

    public void unloadWorld() {
        Bukkit.unloadWorld(this.world, false);
    }

    public void addViewer(Player player) {
        player.setAllowFlight(true);
        player.teleport(world.getSpawnLocation());
    }

    public ItemStack getItem() {
        ItemStack stack = new ItemStack(this.icon);
        ItemMeta meta = stack.getItemMeta();
        meta.displayName(Component.text(name));
        stack.setItemMeta(meta);
        return stack;
    }

    public boolean isEnabled() {
        return enabled;
    }

    public World getWorld() {
        return world;
    }
}
